;; Generated package description from on-screen.el  -*- no-byte-compile: t -*-
(define-package "on-screen" "1.3.3.0.20201128.1411" "guide your eyes while scrolling" '((cl-lib "0")) :keywords '("convenience") :authors '(("Michael Heerdegen" . "michael_heerdegen@web.de")) :maintainer '("Michael Heerdegen" . "michael_heerdegen@web.de") :url "https://github.com/michael-heerdegen/on-screen.el")
